'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var Promise = require('bluebird');
var clc = require('cli-color');

var Client = (function () {

  /**
   * @constructor
   * @param  {RethinkDB} r
   * @param  {Object} config
   */

  function Client(r, config) {
    _classCallCheck(this, Client);

    this.r = r;
    this.config = config;
    this.resultSize = 0;
  }

  /**
   * Convenience method for handling connecting to RethinkDB
   * 
   * @return {Promise}
   */

  _createClass(Client, [{
    key: 'connect',
    value: function connect() {
      return this.r.connect(this.config);
    }
  }, {
    key: 'setDb',
    value: function setDb(val) {
      if (!val) {
        console.log(clc.cyanBright('[x] Current db is: ' + this.config.db));
      } else {
        this.config.db = val;
        console.log(clc.cyanBright('[.] Changed default database to ' + val));
      }

      return Promise.resolve();
    }

    /**
     * Get the available 
     * @return {[type]} [description]
     */
  }, {
    key: 'getTableList',
    value: function getTableList() {
      var _this = this;

      return this.connect().then(function (conn) {
        return _this.r.tableList().run(conn).then(function (x) {
          conn.close();
          return x;
        });
      }).then(function (list) {
        var output = '[-] Available tables in ' + _this.config.db + ' on ' + _this.config.host + ':';
        if (list.length === 0) {
          output += '\n  NONE';
        } else {
          list.forEach(function (db) {
            return output += '\n  - ' + db;
          });
        }

        return output;
      }).then(console.log);
    }

    /**
     * Get the available 
     * @return {[type]} [description]
     */
  }, {
    key: 'getDbList',
    value: function getDbList() {
      var _this2 = this;

      return this.connect().then(function (conn) {
        return _this2.r.dbList().run(conn).then(function (x) {
          conn.close();
          return x;
        });
      }).then(function (list) {
        var output = '[-] Available databases on ' + _this2.config.host + ':';
        if (list.length === 0) {
          output += '\n  NONE';
        } else {
          list.forEach(function (db) {
            return output += '\n  - ' + db;
          });
        }

        return output;
      }).then(console.log);
    }

    /**
     * Evaluate a RethinkDB ReQL expression
     * 
     * @param  {RethinkDB} r
     * @param  {String} query
     * @return {Object|false}
     */
  }, {
    key: 'evaluate',
    value: function evaluate(r, query) {
      try {
        return eval(query);
      } catch (e) {
        return false;
      }
    }

    /**
     * Execute a ReQL query and print out the result
     * 
     * @param  {String} query
     * @return {Promise}
     */
  }, {
    key: 'query',
    value: function query(_query) {
      var _this3 = this;

      var startTime = +new Date();
      var that = this;

      if (_query.indexOf('r.') !== 0) {
        return Promise.reject('[x] Error: Must begin with r.');
      }

      /**
       * Bring in rethinkdb api for local access when eval'ing command
       */
      var r = that.r;

      /**
       * Evaluate query and reject any invalid code
       */
      var eq = that.evaluate(r, _query);
      if (!eq) {
        return Promise.reject('[x] Error: Not supported');
      }

      return this.connect().then(function (conn) {
        return eq.run(conn).then(_this3.handleResponse.bind(_this3)).then(function () {
          return conn.close();
        }).then(function () {
          var endTime = +new Date(),
              diff = (endTime - startTime) / 1000;
          console.log(clc.cyanBright('[.] Took: ' + diff + ' seconds'));
          console.log(clc.cyanBright('[.] No Results: ' + that.resultSize));
          that.resultSize = 0;
        });
      });
    }

    /**
     * Take a response from RethinkDB and print it to console
     * 
     * @param  {Object} cursor
     * @return {void}
     */
  }, {
    key: 'handleResponse',
    value: function handleResponse(cursor) {
      var that = this;
      if (cursor.hasOwnProperty('each')) {
        cursor.each(function (e, x) {
          if (!e) {
            console.log(JSON.stringify(x, null, 2));
            that.resultSize++;
          }
        });
      } else if (cursor.hasOwnProperty('next')) {
        cursor.next(function (e, x) {
          if (!e) {
            console.log(JSON.stringify(x, null, 2));
            that.resultSize++;
          }
        });
      } else {
        console.log(JSON.stringify(cursor, null, 2));
        if (cursor instanceof Array) {
          that.resultSize = cursor.length;
        } else {
          that.resultSize++;
        }
      }
    }
  }]);

  return Client;
})();

exports['default'] = Client;
module.exports = exports['default'];